@@ -134,46 +134,51 @@ class CAM(object):
         worker_start_time = get_time()
         self.worker_id = get_host()
         os.system("tmux rename-window cam%d"%os.getpid())
-        self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Wait Task"]))
         while True:
-            cnt = self._redis.llen("pending")
-            if cnt <= 0:
-                self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Wait Task", cond, cmdprefix, cmdsuffix]))
-            elif not self._condition_parse(cond):
-                self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Wait Resource", cond, cmdprefix, cmdsuffix]))
-            else:
-                row_str = self._redis.rpop("pending")
-                if row_str is None:
-                    continue
-                self.running_job_id, ptime, cmd, status = parse_json(row_str)
-                cmd = "".join([cmdprefix, cmd, cmdsuffix])
-                self._redis.lpush("running", json.dumps([self.running_job_id, get_time(), cmd, get_host()]))
-                log_info("{0} Running task: {1}".format(self.worker_id, self.running_job_id))
-                log_info("{0} Running command: {1}".format(self.worker_id, cmd), )
-                self.p = Popen(cmd, shell=True)
-                worker_start_time = get_time()
-                while True:
-                    self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Running %d"%self.running_job_id, cond, cmdprefix, cmdsuffix]))
-                    tf = self._get_by_tid("running", self.running_job_id)
-                    taskinfo = taskinfo if tf is None else json.dumps(tf) 
-                    self._set_by_tid("running", self.running_job_id, taskinfo)
-                    try:
-                        if self._get_by_tid("running", self.running_job_id)[-1] == "KILLED":
-                            self.p.kill()
-                            log_warn("Task ", str(self.running_job_id), " has been killed.")
+            try:
+                cnt = self._redis.llen("pending")
+                if not hasattr(self, "server_disconnected") or self.server_disconnected:
+                    self.server_disconnected = False
+                    log_info("Server Connected.")
+                if cnt <= 0:
+                    self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Wait Task", cond, cmdprefix, cmdsuffix]))
+                elif not self._condition_parse(cond):
+                    self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Wait Resource", cond, cmdprefix, cmdsuffix]))
+                else:
+                    row_str = self._redis.rpop("pending")
+                    if row_str is None:
+                        continue
+                    self.running_job_id, ptime, cmd, status = parse_json(row_str)
+                    cmd = "".join([cmdprefix, cmd, cmdsuffix])
+                    self._redis.lpush("running", json.dumps([self.running_job_id, get_time(), cmd, get_host()]))
+                    log_info("{0} Running task: {1}".format(self.worker_id, self.running_job_id))
+                    log_info("{0} Running command: {1}".format(self.worker_id, cmd), )
+                    self.p = Popen(cmd, shell=True)
+                    worker_start_time = get_time()
+                    while True:
+                        try:
+                            self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Running %d"%self.running_job_id, cond, cmdprefix, cmdsuffix]))
+                            tf = self._get_by_tid("running", self.running_job_id)
+                            taskinfo = taskinfo if tf is None else json.dumps(tf) 
+                            self._set_by_tid("running", self.running_job_id, taskinfo)
+                            if self._get_by_tid("running", self.running_job_id)[-1] == "KILLED":
+                                self.p.kill()
+                                log_warn("Task ", str(self.running_job_id), " has been killed.")
+                                break
+                        except:
+                            log_warn("Server Disconnected.")
+                            self.server_disconnected = True
+                            time.sleep(10)
+                        try:
+                            self.p.wait(timeout = 10)
                             break
-                    except:
-                        log_warn("Server Disconnected.")
-                        #if not hasattr(self, "server_disconnected"):
-                        #    log_warn("Server Disconnected.")
-                        #    self.server_disconnected = True
-                    try:
-                        self.p.wait(timeout = 10)
-                        break
-                    except:
-                        pass
-                log_info("{0} Finished command: {1}".format(self.worker_id, cmd))
-                self._remove_by_tid("running", self.running_job_id)
+                        except:
+                            pass
+                    log_info("{0} Finished command: {1}".format(self.worker_id, cmd))
+                    self._remove_by_tid("running", self.running_job_id)
+            except:
+                log_warn("Server Disconnected.")
+                self.server_disconnected = True
             time.sleep(5)
             
     def add(self, cmd, order = -1):
@@ -198,31 +203,34 @@ class CAM(object):
         """
         now = datetime.datetime.utcnow()
         log_info("Server: ", self._conf['server'], ":", str(self._conf['port']), ' v', self.__version__)
-        if type is None or type == "task":
-            pending = self._redis.lrange("pending", 0, -1)
-            running = self._redis.lrange("running", 0, -1)
-            res = pending + running
-            nres = []
-            for i in range(len(res)):
-                data = parse_json(res[i])
-                st = datetime.datetime.fromisoformat(data[1])
-                data[1] = str(now - st).split('.')[0]
-                if maxwidth is not None:
-                    data[2] = data[2][:maxwidth]
-                nres.append(data)
-            print(table_list(nres, headers = ["ID", "Time", "Command", "Host/PID"]))
-            print()
-        if type is None or type == "worker":
-            workers = self._redis.hgetall("workers")
-            info = []
-            for w in workers:
-                dt = parse_json(workers[w])
-                st = datetime.datetime.fromisoformat(dt[0])
-                lst = [w, str(now - st).split('.')[0]] + dt[1:]
-                if maxwidth is not None and len(lst) > 3:
-                    lst[3] = lst[3][:maxwidth]
-                info.append(lst)    
-            print(table_list(info, headers = ["Worker/PID", "Up Time", "Status", "cond", "prefix", "suffix"]))
+        try:
+            if type is None or type == "task":
+                pending = self._redis.lrange("pending", 0, -1)
+                running = self._redis.lrange("running", 0, -1)
+                res = pending + running
+                nres = []
+                for i in range(len(res)):
+                    data = parse_json(res[i])
+                    st = datetime.datetime.fromisoformat(data[1])
+                    data[1] = str(now - st).split('.')[0]
+                    if maxwidth is not None:
+                        data[2] = data[2][:maxwidth]
+                    nres.append(data)
+                print(table_list(nres, headers = ["ID", "Time", "Command", "Host/PID"]))
+                print()
+            if type is None or type == "worker":
+                workers = self._redis.hgetall("workers")
+                info = []
+                for w in workers:
+                    dt = parse_json(workers[w])
+                    st = datetime.datetime.fromisoformat(dt[0])
+                    lst = [w, str(now - st).split('.')[0]] + dt[1:]
+                    if maxwidth is not None and len(lst) > 3:
+                        lst[3] = lst[3][:maxwidth]
+                    info.append(lst)    
+                print(table_list(info, headers = ["Worker/PID", "Up Time", "Status", "cond", "prefix", "suffix"]))
+        except:
+            log_warn("Server Disconnected.")
 
     def config(self):
         """
